<?php

class AccessControlManager {
	
	const PASSWORDBACKEND_LOCAL = "local";
	const PASSWORDBACKEND_LDAP = "ldap";
	const PASSWORDBACKEND_AD = "ad";
	
	static $instance;
	private $db;
	
	private $cache;
	
	
	public function __construct() {
		$this->cache = array();
		
		$dbRoot = ServerConfig::getDatabasePath() . "../general/";
		$dbFile = realpath($dbRoot) . "/system.s3db";
		
		if (!file_exists($dbFile)) {
			
			// Check if we are able to create it
 			$isWriteAble = @fopen($dbFile, 'w');
			if ($isWriteAble === false) die("Can't create/write to database file: " . $dbFile);
			fclose($isWriteAble);			
			
			// Create databasefile
			$this->db = new PDO("sqlite:" . $dbFile);
			
			if (!$this->db) {
				die("Can't create new database file: " . $dbFile);
			}
			
			// Read SQL statmens, that create the main structure
			$sql_1 = file_get_contents($dbRoot . "sql/1.sql");
			$sql_2 = file_get_contents($dbRoot . "sql/2.sql");
			$sql_3 = file_get_contents($dbRoot . "sql/3.sql");
			$sql_4 = file_get_contents($dbRoot . "sql/4.sql");
			
			// Execute SQL
			$this->db->exec($sql_1);
			$this->db->exec($sql_2);
			$this->db->exec($sql_3);
			$this->db->exec($sql_4);
				
		} else {
			$this->db = new PDO("sqlite:" . $dbFile);
		}
		
		self::$instance = $this;
	}
	
	
	public function __destruct() {
		
	}
	
	static function getInstance() {
		if (empty(self::$instance)) {
			self::$instance = new AccessControlManager();
		}
		return self::$instance;
	}

	/**
	 * Return all users
	 * 
	 * @return array Returns an array with users
	 */
	public function getUsers() {		
		$users = array();		
		$sql = "SELECT id, username, password, name, division, admin, agreement, templateUser, viewUser, reportGeneratorUser FROM Users ORDER BY name";		
		foreach ($this->db->query($sql) as $row) {
			$users[] = new User($row[0], $row[1], $row[2], $row[3], $row[4], $row[5], $row[6], $row[7], $row[8], $row[9]);
		}
		return $users;		
	}
	
	public function getUser($id = null, $username = null) {	
		if (!empty($id)) {
			$sql = "SELECT id, username, password, name, division, admin, agreement, templateUser, viewUser, reportGeneratorUser FROM Users WHERE id = " . $id;
		} else if (!empty($username)) {
			$sql = "SELECT id, username, password, name, division, admin, agreement, templateUser, viewUser, reportGeneratorUser FROM Users WHERE username = '" . $username . "'";
		} else {
			return null;
		}
		foreach ($this->db->query($sql) as $row) {
			return new User($row[0], $row[1], $row[2], $row[3], $row[4], $row[5], $row[6], $row[7], $row[8], $row[9]);
		}
		return null;
	}
	

	
	public function getGroups() {		
		$groups = array();		
		$sql = "SELECT id, name, description FROM Groups ORDER BY name";		
		foreach ($this->db->query($sql) as $row) {
			$groups[] = new Group($row[0], $row[1], $row[2]);
		}
		return $groups;
	}
	
	public function getGroup($id = null, $name = null) {	

		if (!empty($id)) {
			$sql = "SELECT id, name, description FROM Groups WHERE id = " . $id;
		} else if (!empty($name)) {
			$sql = "SELECT id, name, description FROM Groups WHERE name = '" . $name . "'";
		} else {
			return null;
		}

		foreach ($this->db->query($sql) as $row) {
			return new Group($row[0], $row[1], $row[2]);
		}
		return false;
	}
	
	static function getUserMembers($userId, $recursive = false) {
		self::$instance = self::getInstance();
		
		// check internal cache
		if (isset(self::$instance->cache[$userId][$recursive])) {
			Logger::Debug("AccessControlManager::getUserMembers()", "Reusing internal cache..");
			return self::$instance->cache[$userId][$recursive];
		}
		
		$groups = array();		
		$sql = "SELECT Groups.id, Groups.name, Groups.description FROM Groups, Members WHERE Groups.id = Members.groupId AND Members.type = 1 AND Members.member = $userId";
		foreach (self::$instance->db->query($sql) as $row) {
			$groups[] = new Group($row[0], $row[1], $row[2]);
		}
		
		// If recursive is set, we should find all groups based on the group
		if ($recursive) {
			$allGroups = array();
			foreach ($groups as $group) {
				if (!in_array($group, $allGroups)) {
					$allGroups[] = $group;
				}
				$subGroups = $group->getGroups();
				foreach ($subGroups as $subGroup) {
					if (!in_array($subGroup, $allGroups)) {
						$allGroups[] = $subGroup;
					}
				}
			}
			$groups = $allGroups;
		}
		
		// set internal cache
		self::$instance->cache[$userId] = array();
		self::$instance->cache[$userId][$recursive] = $groups;
		
		return $groups;		
		
	}

	static function getGroupMembers($groupId) {
		self::$instance = self::getInstance();
		
		$groups = array();		
		$sql = "SELECT Groups.id, Groups.name, Groups.description FROM Groups, Members WHERE Groups.id = Members.groupId AND Members.type = 2 AND Members.member = $groupId";
		foreach (self::$instance->db->query($sql) as $row) {
			$groups[] = new Group($row[0], $row[1], $row[2]);
		}
		return $groups;		
		
	}
	
	static function updateUser(&$user) {
		self::$instance = self::getInstance();
		
		if ($user->getId() == "") {
			$sql = "INSERT INTO Users VALUES (" .
			"null, " .
			"'" . $user->getUsername() . "', " .
			"'" . $user->getPassword() . "', " .
			"'" . $user->getName() . "', " .
			"'" . $user->getDivision() . "', " .
			($user->isAdmin() ? 1 : 0) . ", " .
			($user->isAgreement() ? 1 : 0) . ", " .
			($user->isTemplateUser() ? 1 : 0) . ", " .
			($user->isViewUser() ? 1 : 0) . ", " .			
			($user->isReportGeneratorUser() ? 1 : 0) .		
			");";
		} else {
			$sql = "UPDATE Users SET " .
			"username = '" . $user->getUsername() . "', " .
			"password = '" . $user->getPassword() . "', " .
			"name = '" . $user->getName() . "', " .
			"division = '" . $user->getDivision() . "', " .
			"admin = " . ($user->isAdmin() ? 1 : 0) . ", " .
			"agreement = " . ($user->isAgreement() ? 1 : 0) . ", " .
			"templateUser = " . ($user->isTemplateUser() ? 1 : 0) . ", " .
			"viewUser = " . ($user->isViewUser() ? 1 : 0) . ", " .
			"reportGeneratorUser = " . ($user->isReportGeneratorUser() ? 1 : 0) . " " .
			"WHERE id = " . $user->getId() . ";";
		}
		
		if (!self::$instance->db->exec($sql)) return false;
		
		if ($user->getId() == "") {
			//die(self::$instance->db->lastInsertId());
			$user->setId(self::$instance->db->lastInsertId());
		}
		
		return true;
	}
	
	static function removeUser($userId) {
		if (empty($userId)) return false;
		
		self::$instance = self::getInstance();
		
		$sql = "DELETE FROM Users WHERE id = " . $userId . ";";
		if (!self::$instance->db->exec($sql)) return false;
		
		return true;
	}	
	
	static function removeGroup($groupId) {
		if (empty($groupId)) return false;
		
		self::$instance = self::getInstance();
		
		$sql = "DELETE FROM Groups WHERE id = " . $groupId . ";";
		if (!self::$instance->db->exec($sql)) return false;
		
		return true;
	}

	static function updateGroup($group) {
		self::$instance = self::getInstance();
		
		if ($group->getId() == "") {
			$sql = "INSERT INTO Groups VALUES (null, '" . $group->getName() . "', '" . $group->getDescription() . "');";
		} else {
			$sql = "UPDATE Groups SET name = '" . $group->getName() . "', description = '" . $group->getDescription() . "' WHERE id = " . $group->getId() . ";";
			Logger::Debug("AccessControlManager::updateGroup()", "SQL : " . $sql);
		}
		$return = self::$instance->db->exec($sql);
		if ($group->getId() == "") {
			$group->setId(self::$instance->db->lastInsertId());
		}
		return $return;
	}
	
	
	public function updateDataTypesInGroup($group, $datatypeIds) {
		if (!$group instanceof Group || $group->getId() == "") return;
		
		$existingIds = $this->getGroupDataTypes($group->getId(), true);
		
		// Set eveything to not visible first
		$sql = "UPDATE DataTypeAccess SET visible = 0 WHERE groupId = " . $group->getid() . ";";
		$this->db->exec($sql);
		
		foreach ($datatypeIds as $datatypeId) {
			if (in_array($datatypeId, $existingIds)) {
				$sql = "UPDATE DataTypeAccess SET visible = 1 WHERE groupId = " . $group->getid() . " AND datatypeId = " . $datatypeId . ";";
			} else {
				$sql = "INSERT INTO DataTypeAccess VALUES (null, " . $group->getid() . ", " . $datatypeId . ", 1);";
			}
			$this->db->exec($sql);
			Logger::Debug("AccessControlManager::updateDataTypesInGroup()", "SQL : " . $sql);
		}
		
	}
	
	public function updateProductsInGroup($group, $products, $hproducts) {
		if (!$group instanceof Group || $group->getId() == "") return;
		
		$existingProds = $this->getGroupProducts($group->getId(), true);
        $existingVisibleProds = $this->getGroupProducts($group->getId(), false);
		
        if($hproducts){
        	$sql = $this->db->prepare("DELETE FROM ProductAccess WHERE groupId = " . $group->getid() . " AND productName = :product");
			$this->db->beginTransaction();
			foreach ($hproducts as $product) {
				$sql->bindParam(':product', sqlite_escape_string(stripslashes($product)));
				$sql->execute();	 
			}
			$this->db->commit();
			Logger::Debug("AccessControlManager::updateProductsInGroup()", "SQL : " . $sql);
        }

		if($products){
			$sql = $this->db->prepare("INSERT INTO ProductAccess VALUES (null, " . $group->getid() . ", :product, 1)");
			$this->db->beginTransaction();
			foreach ($products as $product) {
				if (!in_array($product, $existingVisibleProds)) {
					$sql->bindParam(':product', sqlite_escape_string(stripslashes($product)));
					$sql->execute();	
				}  
			}
			$this->db->commit();
			Logger::Debug("AccessControlManager::updateProductsInGroup()", "SQL : " . $sql);
		}		
	}
	
	public function updateFeaturesInGroup($group, $features, $hfeatures) {
		if (!$group instanceof Group || $group->getId() == "") return;
		
		$existingFeats = $this->getGroupFeatures($group->getId(), true);
        $existingVisibleFeats = $this->getGroupFeatures($group->getId(), false);
		
        if($hfeatures){
        	$sql = $this->db->prepare("DELETE FROM FeatureAccess WHERE groupId = " . $group->getid() . " AND featureName = :feature");
			$this->db->beginTransaction();
			foreach ($hfeatures as $feature) {
				$sql->bindParam(':feature', sqlite_escape_string(stripslashes($feature)));
				$sql->execute();
			}
			$this->db->commit();
			Logger::Debug("AccessControlManager::updateFeaturesInGroup()", "SQL : " . $sql);
        }
		
		if($features){
			$sql = $this->db->prepare("INSERT INTO FeatureAccess VALUES (null, " . $group->getid() . ", :feature, 1)");
			$this->db->beginTransaction();
			foreach ($features as $feature) {
				if (!in_array($feature, $existingVisibleFeats)) {
					$sql->bindParam(':feature', sqlite_escape_string(stripslashes($feature)));
					$sql->execute();	
				}  
			}
			$this->db->commit();
			Logger::Debug("AccessControlManager::updateFeaturesInGroup()", "SQL : " . $sql);
		}	
	}

	// AccessControlManager::setAllHide - Insert all Product/Feature from ClassificationValue to Product/FeatureAccess for specified groupID 
	public function setAllHide($group, $values, $cvtype){
		if (!$group instanceof Group || $group->getId() == "") return;

		if($cvtype == "Feature"){
			$existingVisible = $this->getGroupFeatures($group->getId(), false);
		}
		else{
			$existingVisible = $this->getGroupProducts($group->getId(), false);
		}


		if(count($existingVisible) != count($values)){
			$this->setAllUnHide($group, $cvtype);
			$sql = $this->db->prepare("INSERT INTO " . $cvtype . "Access VALUES (null, " . $group->getid() . ", :value , 1)");
			$this->db->beginTransaction();
			foreach ($values as $value) {
				$sql->bindParam(':value', sqlite_escape_string(stripslashes($value)));
				$sql->execute();	 
			}
			$this->db->commit();
			Logger::Debug("AccessControlManager::setAllHide()", "SQL : " . $sql);
		}
	}

	// AccessControlManager::setAllUnHide - Delete all Product/FeatureAccess for specified groupID 
	public function setAllUnHide($group, $cvtype){
		if (!$group instanceof Group || $group->getId() == "") return;
		
		$sql = "DELETE FROM " . $cvtype . "Access WHERE groupId = " . $group->getid() . ";";
        $this->db->exec($sql);
        Logger::Debug("AccessControlManager::setAllUnHide()", "SQL : " . $sql);
	}
	
	public function updateFoldersInGroup($group, $folders) {
		if (!$group instanceof Group || $group->getId() == "") return;
		
		// Remove all groups first, then add all again
		$sql = "DELETE FROM FolderAccess WHERE groupId = " . $group->getid() . ";";
		$this->db->exec($sql);
		
		foreach ($folders as $folder) {
			$sql = "INSERT INTO FolderAccess VALUES (null, " . $group->getid() . ", '" . $folder. "');";
			$this->db->exec($sql);
		}
	}
	
	public function deleteFolder($folder) {
		// Remove all groups first, then add all again
		$sql = "DELETE FROM FolderAccess WHERE folder = '" . $folder->getName() . "';";
		$this->db->exec($sql);
	}
	
	public function getGroupDataTypes($groupId, $all = true) {
		$existingIds = array();
		if ($all) {
			$sql = "SELECT datatypeId FROM DataTypeAccess WHERE groupId = " . $groupId;
		} else {
			$sql = "SELECT datatypeId FROM DataTypeAccess WHERE visible = 1 AND groupId = " . $groupId;
		}
		foreach (self::$instance->db->query($sql) as $row) {
			$existingIds[] = $row[0];
		}
		return $existingIds;		
	}
	
	public function getGroupProducts($groupId, $all = true) {
		$existingProds = array();
		if ($all) {
			$sql = "SELECT productName FROM ProductAccess WHERE groupId = " . $groupId;
		} else {
			$sql = "SELECT productName FROM ProductAccess WHERE visible = 1 AND groupId = " . $groupId;
		}
		
		foreach (self::$instance->db->query($sql) as $row) {
			$existingProds[] = $row[0];
		}
		return $existingProds;		
	}

	public function getGroupFeatures($groupId, $all = true) {
		$existingFeats = array();
		if ($all) {
			$sql = "SELECT featureName FROM FeatureAccess WHERE groupId = " . $groupId;
		} else {
			$sql = "SELECT featureName FROM FeatureAccess WHERE visible = 1 AND groupId = " . $groupId;
		}
		
		foreach (self::$instance->db->query($sql) as $row) {
			$existingFeats[] = $row[0];
		}
		return $existingFeats;		
	}
	
	public function getGroupFolder($groupId) {
		$folders = array();
		$sql = "SELECT folder FROM FolderAccess WHERE groupId = " . $groupId;
		foreach (self::$instance->db->query($sql) as $row) {
			$folders[] = $row[0];
		}
		return $folders;
	}
	
	public function removeMembersFromUser($user) {
		if (!$user instanceof User || $user->getId() == "") return;
		$sql = "DELETE FROM Members WHERE Members.type = 1 AND Members.member = " . $user->getId();
		$this->db->exec($sql);
	}
	
	public function addMembersFromUser($user, $groupIds) {
		if ($user->getId() == "") return;

		// Remove all members first
		$sql = "DELETE FROM Members WHERE type = 1 AND member = " . $user->getId() . "\n";
		$this->db->exec($sql);
		
		$sql = "BEGIN;\n";
		foreach ($groupIds as $groupId) {
			$sql .= "INSERT INTO Members VALUES (null, " . $groupId . ", 1, " . $user->getId() . ");\n";
		}
		$sql .= "COMMIT;\n";
		$this->db->exec($sql);		
	}

	public function addMembersFromGroup($group, $groupIds) {
		if ($group->getId() == "") return;
		
		// Remove all members first
		$sql = "DELETE FROM Members WHERE type = 2 AND member = " . $group->getId() . "\n";
		$this->db->exec($sql);
		
		$sql = "BEGIN;\n";
		foreach ($groupIds as $groupId) {
			$sql .= "INSERT INTO Members VALUES (null, " . $groupId . ", 2, " . $group->getId() . ");\n";
		}
		$sql .= "COMMIT;\n";
		$this->db->exec($sql);
	}
	
	function updateAnonymousClassificationInGroup($group, $datatypeId, $classificationId, $checked) {

		if ($checked) {
			$sql = "INSERT INTO AnonymousAccess VALUES (null, " . $group->getId() . ", " . $datatypeId . ", " . $classificationId . ");\n";
		} else {
			$sql = "DELETE FROM AnonymousAccess WHERE groupId = " . $group->getId() . " AND datatypeId = " . $datatypeId . " AND classificationId = " . $classificationId . ";\n";
		}
		$this->db->exec($sql);
		//echo $sql;
	}
	
	function getAnonymousClassificationInGroup($group) {
		self::$instance = self::getInstance();
		$anonymousClassifications = array();
		
		$sql = "SELECT datatypeId, classificationId FROM AnonymousAccess WHERE groupId = '" . $group->getId() . "'";
		foreach (self::$instance->db->query($sql) as $row) {
			if (!isset($anonymousClassifications[$row[0]])) {
				$anonymousClassifications[$row[0]] = array();
			}
			$anonymousClassifications[$row[0]][] = $row[1];
		}
		return $anonymousClassifications;
	}
	
	function getContentLimit($group) {
		self::$instance = self::getInstance();
		
		$contentLimits = array();		

		$sql = "SELECT datatypeId, value FROM ClassificationValuesAccess WHERE groupId = '" . $group->getId() . "'";
		foreach (self::$instance->db->query($sql) as $row) {
			if (!isset($anonymousClassifications[$row[0]])) {
				$anonymousClassifications[$row[0]] = array();
			}
			$anonymousClassifications[$row[0]][] = $row[1];
		}
				
		return $contentLimits;
	}
	
	
	/* Check if a password match a user profile */
	public static function isPasswordCorrect(User &$user, $password) {
		
		$userPasswordMatch = false;
		
		switch (ServerConfig::getPasswordBackend()) {
			
			case AccessControlManager::PASSWORDBACKEND_AD:				
				$ldapSearcher = new LDAPSearcher();
				$ldapSearcher->setAd(true);
				$ldapSearcher->setServer("192.168.7.10");
				$ldapSearcher->setSuffix("@svg.openit.local");
				$ldapSearcher->setBaseDn("dc=svg,dc=openit,dc=local");
				$ldapSearcher->setUsername($user->getUsername());
				$ldapSearcher->setPassword($password);
				
				if ($ldapSearcher->connect() && $ldapSearcher->authorize()) {
					$searchUser = $ldapSearcher->getUser($user->getUsername());
					$user->setName($searchUser->getName());					
					return true;
				}				
				break;
				
			case AccessControlManager::PASSWORDBACKEND_LDAP:
				break;
				
			case AccessControlManager::PASSWORDBACKEND_LOCAL:
				if ($user->getPassword() == md5($password)) {
					return true;
				}
				break;
				
		}
		
		// If user is not accepeted, and using something else than local and is admin, we try local as well
		if (!$userPasswordMatch &&
			ServerConfig::getPasswordBackend() !=  AccessControlManager::PASSWORDBACKEND_LOCAL &&
			$user->getUsername() == "admin"
			) {
			$userPasswordMatch = $user->getPassword() == md5($password) ? true : false;
		}
		
		return $userPasswordMatch;
	}
	
	/* Check if a password match a user profile */
	public static function getLDAPUser($username) {
		
		switch (ServerConfig::getPasswordBackend()) {
			
			case AccessControlManager::PASSWORDBACKEND_AD:				
				$ldapSearcher = new LDAPSearcher();
				$ldapSearcher->setAd(true);
				$ldapSearcher->setServer("192.168.7.10");
				$ldapSearcher->setSuffix("@svg.openit.local");
				$ldapSearcher->setBaseDn("dc=svg,dc=openit,dc=local");
				$ldapSearcher->setUsername("cerberus");
				$ldapSearcher->setPassword("Hvd0bd#5");
				
				if ($ldapSearcher->connect() && $ldapSearcher->authorize()) {
					return $ldapSearcher->getUser($username);
				}
				return null;				
				break;
				
			case AccessControlManager::PASSWORDBACKEND_LDAP:
				return null;
				break;
				
			case AccessControlManager::PASSWORDBACKEND_LOCAL:
			default:
				return null;
				break;
		}
		return null;				
	}
	
	
}

?>