<?php

class DatabaseManager {
	
	private $db;
	private $user;
	static private $dataInformation;
	
	static private $static;
	
	function __construct() {
		$currentDatabase = realpath(ServerConfig::getDatabase());
		Logger::Debug("{CLI:DatabaseManager.php}", "The database is  " . $currentDatabase);
		if ($currentDatabase == "" || !is_file($currentDatabase)) {
			if ($_SESSION["USER"]->isAdmin()) {
				Logger::Error("DataLoader:__constructor", "Can’t load the database file! Since an administrator is logged in, we start generating data..");
				
				// Load presentation layer class
				$presentation = new Presentation();
		
				// Show presentation
				$presentation->display("pages/firsttime.php");
				
				// Make sure that the content is displayed on screen
				ob_flush();
				flush();		
				
				$config = new ConfigReader(ServerConfig::getServerUrl() . 'config_dump.pl?version=5.4');
				$config->parse();
				ConsoleLogger::write($presentation->getText("consolelogger_messages_configuration_refreshed"));
				Logger::Info("{CONFIG}", "The database has been updated");
				
				// Try to keep the user
				$user = $_SESSION["USER"];
				@session_destroy();
				$_SESSION["USER"] = $user;
				?>
					<script type="text/javascript" language="JavaScript">
						var updateing = document.getElementById("message");
						updateing.innerHTML = "<?= htmlspecialchars($presentation->getText("firsttime_done")) ?>";
						setTimeout("top.location = '/index.php'", 5000);
					</script>
				<?php
				exit();
			} else {
				Logger::Error("DataLoader:__constructor", "Can’t load the database file! Stopping program!!");
				session_destroy();
				die("ERROR: 729451: Can’t load data, please contact your Open iT administrator!");
			}
		}
		
		$this->db = new PDO("sqlite:" . ServerConfig::getDatabase());
		
		if (!$this->db) {
			Logger::Error("DataLoader:__constructor", "Can’t load the database file! Stopping program!!");
			session_destroy();
			die("ERROR: 729451: Can’t load data, stopping program!");
		}
		
		// Caching information in session, when its not too much, and it’s often used
		self::$dataInformation = array();
		
		if (!isset($_SESSION["DATALOADER_DATAINFORMATION"])) {

			$dataInformations = file(ServerConfig::getServerUrl() . "get_data_dir.pl");
			
			// No data valid for the system!
			// TODO: Need to give information to user!!
			if (strstr(implode("", $dataInformations), "WARNING")) {
				$_SESSION["DATALOADER_DATAINFORMATION"] = array();
				return;
			}
			
			foreach ($dataInformations as $dataInformation) {
				list ($id, $from, $to) = explode(":", $dataInformation);
				self::$dataInformation[$id] = array("FROM" => $from, "TO" => $to);
			}
			
			$_SESSION["DATALOADER_DATAINFORMATION"] = self::$dataInformation;
			
			
		} else {
			self::$dataInformation = $_SESSION["DATALOADER_DATAINFORMATION"];
		}
		
		//print_r($_SESSION["DATALOADER_DATAINFORMATION"]);

	}
	
	private static function checkStatic() {
		if (empty(self::$static)) {
			self::$static = new DatabaseManager();
			self::$static->user = $_SESSION["USER"];
		}
	}
	
	/**
	 * Return all Modules
	 *
	 * @return array Module
	 */
	static public function getModules() {

		self::checkStatic();

		$modules = array();
		foreach (self::$static->db->query("SELECT id, value FROM Module ORDER BY value") as $row) {
				$modules[] = new Module($row[0], $row[1]);
		}
		return $modules;
	}

	
	
	/**
	 * Return all Data Types
	 *
	 * @return array DataType
	 */
	static public function getDataTypes() {

		self::checkStatic();

		$dataTypes = array();
		foreach (self::$static->db->query("SELECT id, value, moduleId FROM DataType ORDER BY value") as $row) {
				if (isset(self::$dataInformation[$row[0]])) {
					if (!self::$static->user->isAdmin() && !self::$static->user->isMemberOfDataType($row[0])) {
						continue;
					}
					$dataTypes[] = new DataType($row[0], ($row[0]-1), $row[1], self::$dataInformation[$row[0]]["FROM"], self::$dataInformation[$row[0]]["TO"], $row[2]);
				}
		}
		return $dataTypes;
	}
	

	
	/**
	 * Return a DataType
	 *
	 * @param int $id The ID of the DataType
	 * @return DataType
	 */
	static public function getDataType($id) {
		self::checkStatic();
		foreach (self::$static->db->query("SELECT id, value FROM DataType WHERE id = $id") as $row) {
			if (isset(self::$dataInformation[$row[0]])) {
				$dataFrom = self::$dataInformation[$row[0]]["FROM"];
				$dataTo = self::$dataInformation[$row[0]]["TO"];
			} else {
				$dataFrom = "N/A";
				$dataTo = "N/A";
			}
			return new DataType($row[0], ($row[0]), $row[1], $dataFrom, $dataTo);
		}
		return false;
	}
	
	static public function getMeasurements(DataType $dataType) {
		self::checkStatic();		
		$measurements = array();
		foreach (self::$static->db->query("SELECT id, internalId, value, description FROM Measurement WHERE datatypeId = " . $dataType->getUID()) as $row) {
			$measurements[] = new Measurement($row[0], $row[1], $row[2], $row[3], $dataType);
		}
		return $measurements;
	}
	

	static public function getClassifications(DataType $dataType) {
		self::checkStatic();		
				
		$classifications = array();

		// Always return "Time" as an classification
		$classifications[] = new Classification("T", "T", "Time", 1, null, $dataType, 0);

		foreach (self::$static->db->query("SELECT id, internalId, value, static, anonymous, virtualGui FROM Classification WHERE datatypeId = " . $dataType->getUID()) as $row) {
			$classifications[] = new Classification($row[0], $row[1], $row[2], $row[3], $row[4], $dataType, $row[5]);
		}

		return $classifications;
	}
	
	
	
	static public function getClassificationValues(Classification $classification) {
		
		if (!$classification) die("Auu");
		
		self::checkStatic();
		
		$classificationValues = array();
		$selectedClassificationValues = $classification->getMarkedClassificationValues();
		
		// better look up
		$selectedClassificationValuesNEW = array();
		foreach ($selectedClassificationValues as $selectedClassificationValueNew) {
			$selectedClassificationValuesNEW[$selectedClassificationValueNew] = 1;
		}
		
		foreach (self::$static->db->query("SELECT id, value, anonymous, static FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . " ORDER BY value") as $row) {
			//$selected = in_array($row[0], $selectedClassificationValues) ? true : false;
			$selected = isset($selectedClassificationValuesNEW[$row[0]]) ? true : false;
			$classificationValues[] = new ClassificationValue($row[0], $row[1], $row[2], $selected, $classification);
		}
		return $classificationValues;		
	
		
	}
	
	/*
	 * Lists all distinct values
	 */
	static public function getAllClassificationValues($classification, $keyword = "") {
		$clValues = array();
		foreach(self::$static->db->query("SELECT DISTINCT cv.value FROM Classification AS c, ClassificationValue AS cv WHERE c.value='". $classification ."' AND c.Id=cv.classificationId AND cv.value LIKE '%".$keyword."%' ORDER BY cv.value") as $row){
			$clValues[] = $row[0];
		}
		return $clValues;
	}
        
        /*
	 * Lists all distinct values within the range
	 */
	static public function getAllClassificationValuesWithOffset($classification, $offset, $count, $keyword = "") {
		$clValues = array();
		foreach(self::$static->db->query("SELECT DISTINCT cv.value FROM Classification AS c, ClassificationValue AS cv WHERE c.value='". $classification ."' AND c.Id=cv.classificationId AND cv.value LIKE '%".$keyword."%' ORDER BY cv.value LIMIT ".$offset. "," .$count) as $row){
			$clValues[] = $row[0];
		}
		return $clValues;
	}

	/*
	 * Lists all allowed values
	 */
	static public function getAllowedClassificationValues($classification ) {
		$clValues = array();
		foreach(self::$static->db->query("SELECT DISTINCT cv.value FROM Classification AS c, ClassificationValue AS cv WHERE c.value='". $classification ."' AND c.Id=cv.classificationId ORDER BY cv.value") as $row){
			if(self::$static->user->isMemberOfProducts($row[0])){
				$clValues[] = $row[0];
			}
		}
		
		return $clValues;
	}
	
	// Sortering
	
	static public function getRelations(Classification $classification) {
		self::checkStatic();
		
		$SQL = "SELECT C.id FROM Classification AS C WHERE C.id IN (SELECT DISTINCT CV.classificationId FROM ClassificationValue AS CV WHERE CV.id IN (SELECT VR.leftValueId FROM ValuesRelationship AS VR WHERE VR.rightValueId IN (SELECT CV.id FROM ClassificationValue AS CV WHERE CV.classificationId = ||ID||) UNION SELECT VR.rightValueId FROM ValuesRelationship AS VR WHERE VR.leftValueId IN (SELECT CV.id FROM ClassificationValue AS CV WHERE CV.classificationId = ||ID||)))";
		$SQL = str_replace("||ID||", $classification->getUID(), $SQL);
			
		$relationIds = array();
		foreach (self::$static->db->query($SQL) as $row) {
			$relationIds[] = $row[0];
		}
		return $relationIds;
	}

	static public function getRelationTree(DataType $dataType) {
		self::checkStatic();
        logger::Debug(__METHOD__, "Generating relation tree");
		$relation = array();
		
		foreach ($dataType->getClassifications() as $classification) {
			
			// Make sure that we only proccess valid IDs (Time classification has the static ID "T")
			if (!is_numeric($classification->getID())) continue;
			
			$relation[$classification->getUID()] = array();
			
			foreach (self::getRelations($classification) as $relationId) {
				$relation[$classification->getUID()][$relationId] = true;
			}
		}
		logger::Debug(__METHOD__, "Returning relation tree");
		return $relation;
	}
	
	
	/**
	 * New and a smarter way of getting values
	 * 
	 * @param $classification The classification object where you want to have the values from
	 * @return array ClassificationValues
	 */
	static public function getValuesNew2(Classification $classification) {
		
		self::checkStatic();
		$classificationValues = array();
        //$classificationFilter = wgDbClassificationFilter::getSql();
		
		// COP ONLY? CASE SENSETIVE LIKE!
		self::$static->db->exec('PRAGMA case_sensitive_like = 1');
				
		// Check if we have any selections, if not, return all values
		if (!isset($_SESSION["NEW_STAT"]) || !is_array($_SESSION["NEW_STAT"]) || count($_SESSION["NEW_STAT"]) == 0) {

			Logger::Debug(__CLASS__, "SQL: SELECT id, value, anonymous, static FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . " ORDER BY value");
			Logger::Debug(__CLASS__, 'Count1: ' . self::$static->db->query("SELECT count(*) FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . " ORDER BY value")->fetchColumn());
            //Logger::Debug(__CLASS__, 'Count2: ' . self::$static->db->query("SELECT count(*) FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . " {$classificationFilter} ORDER BY value")->fetchColumn());
			            
            //$results = self::$static->db->query("SELECT id, value, anonymous, static FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . " {$classificationFilter} ORDER BY value");
			$results = self::$static->db->query("SELECT id, value, anonymous, static FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . "  AND value NOT LIKE 'Slb_Internal%' AND value NOT LIKE 'Petrel_Bo%' ORDER BY UPPER(value)"); // COP Only!
			
			// COP ONLY
			// Build exclude list
			$copHide = " AND value NOT LIKE 'Slb_Internal%' AND value NOT LIKE 'Petrel_Bo%' ";			
			if (file_exists('/opt/openit/config/hide')) {
				foreach (file('/opt/openit/config/hide') as $hideValue) {
					$hideValue = trim($hideValue);
					$copHide .= "AND value NOT LIKE '{$hideValue}' ";
				}				
			}
		
			$results = self::$static->db->query("SELECT id, value, anonymous, static FROM ClassificationValue WHERE classificationId = " . $classification->getUID() . $copHide . " ORDER BY UPPER(value)"); // COP Only!
			

			Logger::Debug(__CLASS__, "All values has been recived");
			foreach ($results as $row) {
				$classificationValues[] = new ClassificationValue($row[0], $row[1], $row[2], false, $classification);	
			}
			Logger::Debug(__CLASS__, "All values has been pre-proccessed");
			return $classificationValues;
		}
		

		// Advanced SQL query
		Logger::Debug(__METHOD__, "Running advanced querying...", __LINE__);
		Logger::Debug(__METHOD__, "Generating the SQL", __LINE__);
		$SQL = SearchGenerator::generateSql($classification->getUID(), SearchBuilder::getSearchGroups($_SESSION["NEW_STAT"]), self::getRelationTree($classification->getDataType()));
		Logger::Debug(__METHOD__, "SQL: " . $SQL, __LINE__);
		
		Logger::Debug(__METHOD__, "Execute SQL", __LINE__);
		foreach (self::$static->db->query($SQL) as $row) {
			$selected = (isset($_SESSION["NEW_STAT"][$row[0]])) ? true : false;			
			$classificationValues[] = new ClassificationValue($row[0], $row[1], $row[2], $selected, $classification);
		}
		Logger::Debug(__METHOD__, "Done executeing SQL", __LINE__);
		
		return $classificationValues;
	}
	
	
	
	
	/**
	 * Generating SQL subquries
	 * 
	 * @param $dest Classification ID that you want to return
	 * @param $src Classification ID that has a value set
	 * @param $value Classification Value ID that is set
	 * @return string Returns a SQL string 
	 */
	static public function generateSubQuery($dest, $src, $value, $type = "AND") {

		// Check if the classification is directly connected to us..	
		if (isset($_SESSION["RELATIONS"][$dest][$src])) {

$SQL = <<<EOT
	$type (
	        	id IN (SELECT leftValueId FROM ValuesRelationship WHERE rightValueId = $value)
		OR 	id IN (SELECT rightValueId FROM ValuesRelationship WHERE leftValueId = $value)
  	)
EOT;
			return $SQL;
		}

		// if there is no directly connection, we need to find the connection
		// this is done by using the relation table
		else {

			// Try first, if none, try next unitil its failing	
			$_SESSION["done"][$dest] = 1;

			// TODO: find shortest way
			foreach($_SESSION["RELATIONS"][$dest] as $key => $null) {
				if (isset($_SESSION["done"][$key])) continue;			
	
				$data = self::generateSubQuery($key, $src, $value, "AND");
				if (trim($data) == "") continue;
		
$SQL = <<<EOT
	$type (
			id IN (
				SELECT leftValueId FROM ValuesRelationship WHERE rightValueId IN ( 
					SELECT id FROM ClassificationValue WHERE classificationId = $key
$data 
				)
			)
		OR	id IN (
				SELECT rightValueId FROM ValuesRelationship WHERE leftValueId IN ( 
					SELECT id FROM ClassificationValue WHERE classificationId = $key
$data
				)
			)
	)
EOT;
				return $SQL;
			}
		}		
	}
	
	
	static public function getValuesNew(Classification $classification) {
		
		// Get data type from classification (parent)
		$dataType = $classification->getDataType();
		
		self::checkStatic();
		
		// values to be returned
		$classificationValues = array();
		
		
		Logger::Debug("DatabaseManager::getValuesNew", "Getting classifications for: " . $classification->getValue());
	
		// Get values from relations
		//$relationIds = $classification->relations;
		$relationIds = self::getRelations($classification);
		Logger::Debug("DatabaseManager::getValuesNew", "Total direct relations: " . count($relationIds));
		
		if (is_array($relationIds) && count($relationIds) != 0) {
		
			Logger::Debug("DatabaseManager::getValuesNew", "Getting information from relations");
				
			// Create a limit list
			$limitList = array();
			
			foreach ($relationIds as $relationId) {
				
				// Classification relation object
				$classificationRealtion = $dataType->getClassification($relationId, true);
				if (!$classificationRealtion) continue;
				Logger::Debug("DatabaseManager::getValuesNew", "Get values from relation: " . $classificationRealtion->getValue());				
				
				// Sjekker om Classification har verdier
				if ($classificationRealtion->hasValueSelected()) {
				
					// Hent verdier til valgt classifications
					$values = $classificationRealtion->getClassificationValues(false);
					Logger::Debug("DatabaseManager::getValuesNew", "The number of returned values from realtion was: " . count($values));
					
					
					
					
					
					// If there are values returned, add them to my limit list
					if (!empty($values)) {

						// Add holder
						$limitList[$classificationRealtion->getUID()] = array();
						
						// Add ids to holder
						foreach ($values as $value) {
							$limitList[$classificationRealtion->getUID()][] = $value->getUID();
						}
					}
					
				}
				
				
			}

			// use the limit list to limit the values for this classification
			$classificationValues = self::getClassificationValuesFromLimit($classification, $limitList);
		}		
		
		// If there is no classification values, return all
		if (count($classificationValues)==0) {
			return self::getClassificationValues($classification);
		}

		Logger::Debug("DatabaseManager::getValuesNew", "-END-");
		return $classificationValues;
		
		
	}	
	
	
	
	static private function getClassificationValuesFromLimit($classification, $limits) {
		
		$id = $classification->getUID();
		
		//ksort($limits);
		
		if (empty($limits) || !is_array($limits) || count($limits) == 0) {
			return array();
		}
		$classificationValuesReturn = array();
		
		foreach ($limits as $limit2)	{
			sort($limit2);
			
			$where = "";
			$int = 0;
			$MAX = 1000;
			
			$last = null;
			$lastInt = 0;
			
			foreach ($limit2 as $limit)	{

				if (is_null($last)) {
					$last = $limit;
					continue;
				}

				$int++;
				if ($last + $int == $limit) {
					$lastInt++;
					continue;
				}				
				
				if ($lastInt != 0) {
					// Print BETWEEN first, then 
					if ($where != "") $where .= " OR ";
					$where .= " VR.leftValueId BETWEEN " . $last . " AND " . ($last+$lastInt) . " ";
				}
				
				// kun en i last, pint vanlig
				else {
					if ($where != "") $where .= " OR ";
					$where .= " VR.leftValueId = '" . $last . "' ";
				}
				
				// sett ny som last
				$last = $limit;
				$lastInt = 0;
				$int = 0;
				
			}
			
			// adding last.. rule
			if (!is_null($last)) {
				if ($where != "") $where .= " OR ";
				if ($lastInt != 0) {
					$where .= " VR.leftValueId BETWEEN " . $last . " AND " . ($last+$lastInt) . " ";
				} else {
					$where .= " VR.leftValueId = '" . $last . "' ";
				}
			}
			
			
			
			$SQL = "SELECT DISTINCT id, value, anonymous, static FROM ClassificationValue WHERE id IN ( SELECT VR.rightValueId AS ID FROM ValuesRelationship AS VR WHERE (" . $where;
			$SQL .= ") UNION SELECT VR.leftValueId AS ID FROM ValuesRelationship AS VR WHERE ( " . str_replace("leftValueId", "rightValueId", $where) . "))";
			$SQL .= "AND classificationId = '" . $id . "';\n";
					
			Logger::Debug("DatabaseManager::getClassificationValuesFromLimit", "SQL: $SQL");
			
			$selectedClassificationValues = $classification->getMarkedClassificationValues();
			
			$selectedClassificationValuesNEW = array();
			foreach ($selectedClassificationValues as $selectedClassificationValueNew) {
				$selectedClassificationValuesNEW[$selectedClassificationValueNew] = 1;
			}

			$classificationValues = array();
			$result = self::$static->db->query($SQL);

			if ($result) {
				foreach ($result as $row) {
					//$selected = in_array($row[0], $selectedClassificationValues) ? true : false;
					$selected = isset($selectedClassificationValuesNEW[$row[0]]) ? true : false;
					$classificationValues[] = new ClassificationValue($row[0], $row[1], $row[2], $selected, $classification);
				}
			}
			
			// empty, return null - or should it remove everything?
			if (count($classificationValues) == 0) continue;
			
			// If this is the first returned realtion values, add them to return list
			if (count($classificationValuesReturn) == 0) {
				$classificationValuesReturn = $classificationValues;
			}
			
			// if its not the first, we need to keep the values that exists in both results, everything else, we skip
			// since each classification value has the same parent, we need to check on the id
			else {
				
				// Keep the old IDs
				$oldClassificationValues = array();
				foreach ($classificationValuesReturn as $oldClassificationValue) {
					$oldClassificationValues[] = $oldClassificationValue->getUID();
				}				
				
				// Reset the classification values to return
				$classificationValuesReturn = array();
								
				$oldClassificationValuesNew = array();
				foreach ($oldClassificationValues as $oldClassificationValueNew) {
					$oldClassificationValuesNew[$oldClassificationValueNew] = 1;
				}
				
				
				// Add the classification values that exist both in this return, and in the old list of ids
				foreach ($classificationValues as $value) {
					//if (in_array($value->getUID(), $oldClassificationValues)) {
					if (isset($oldClassificationValuesNew[$value->getUID()])) {
						$classificationValuesReturn[] = $value;
					}
				}
		
			}
			
		}
		
		Logger::Debug("DatabaseManager::getClassificationValuesFromLimit", "Number of values found: " . count($classificationValuesReturn));
		return $classificationValuesReturn;
	}
	
	
	
	static public function getClassificationValuesFromSelection(Classification $classification) {
		
		self::checkStatic();
		
		Logger::Debug("DataLoader::getClassificationValuesFromSelection()", "Getting values for classification: " . $classification->getValue());
		
		$selectedIds = $classification->getMarkedClassificationValues();
	
		if (!(isset($selectedIds) && is_array($selectedIds) && count($selectedIds) != 0)) {
			Logger::Debug("DataLoader::getClassificationValuesFromSelection()", "Returning with NULL values");
			return array();
		}
		
		Logger::Debug("DataLoader::getClassificationValuesFromSelection()", "Marked values found: " . count($selectedIds));
		
		$SQL = "";
		$int = 0;
		$MAX = 30000;
		foreach ($selectedIds as $line => $selectedId) {
			if ($line == 0)  {
				$SQL .= $selectedId;
				continue;
			}
			
			if ($int == $MAX) {
				$SQL .= " ) OR ( id = " . $selectedId;
				$int = 0;
				continue;
			}
			
			$int++;
			$SQL .= " OR id = " . $selectedId;
		}
		
		Logger::Debug("DataLoader::getClassificationValuesFromSelection()", "SQL: " . $SQL);
		
		$classificationValues = array();
		foreach (self::$static->db->query('SELECT id, value, anonymous, static FROM ClassificationValue WHERE (id = ' . $SQL . ' ) ORDER BY value;') as $row) {
			$classificationValues[] = new ClassificationValue($row[0], $row[1], $row[2], true, $classification);
		}
		
		Logger::Debug("DataLoader::getClassificationValuesFromSelection()", "Aqtual values returned: " . count($classificationValues));
		
		Logger::Debug("DataLoader::getClassificationValuesFromSelection()", "-END-");
		
		return $classificationValues;
	}

	
}

?>